

package HTTP::DAVServer::MOVE;

our $VERSION=0.1;

use strict;
use warnings;

use File::Copy qw(move);

=head1 NAME

HTTP::DAVServer::MOVE - Implements the MOVE method

=cut


sub handle {

    my ($self, $r, $url, $path, $responder, $request) = @_;

    my $dest = $ENV{'HTTP_DESTINATION_PATH_TRANSLATED'};
    if (! $dest) {
	$dest = $ENV{'HTTP_DESTINATION'};
	$dest =~ s#^http://[^/]*##;
	$dest = $HTTP::DAVServer::ROOT . $dest;
    }

    if ( -d $path ) {
	my @responses = moveDir( $path, $url, $dest );
	if (scalar @responses > 0) {
    	    $responder->multiStatus( $r, join "", @responses );
	}
	
	$responder->created( $r );
    }

    if ( -f $path ) {
        unless (moveFile( $path, $url, $dest ) ) {
	    $responder->created( $r );
	}

        $responder->forbidden( $r );
    }

    $responder->notFound( $r );

}

sub moveFile {

    my ( $path, $url, $dest ) = @_;

    my $response;
    unless (move $path, $dest) {
	$response = qq(<response>\n<href>$HTTP::DAVServer::HOST$url</href>\n<status>403 Forbidden</status>\n</response>);
    }

    return $response;

}

sub moveDir {

    my ( $directory, $url, $dest ) = @_;

    my @responses=();
    opendir MYDIR, $directory;

    while (my $entry = readdir MYDIR) {

        next if $entry =~ /^\.\.?$/;

        if ( -f "$directory/$entry" ) {
            push @responses, moveFile( "$directory/$entry", "$url/$entry", "$dest/$entry" );
        } else {
            push @responses, moveDir( "$directory/$entry", "$url/$entry", "$dest/$entry" );
        }

    }

    warn "MOVE $directory\n" if $HTTP::DAVServer::TRACE;
    unless (mkdir $directory) {
	push @responses, qq(<response>\n<href>$HTTP::DAVServer::HOST$url</href>\n<status>403 Forbidden</status>\n</response>);
    }

    return @responses;

}

=head1 SUPPORT

For technical support please email to jlawrenc@cpan.org ... 
for faster service please include "HTTP::DAVServer" and "help" in your subject line.

=head1 AUTHOR

 Jay J. Lawrence - jlawrenc@cpan.org
 Infonium Inc., Canada
 http://www.infonium.ca/

=head1 COPYRIGHT

Copyright (c) 2003 Jay J. Lawrence, Infonium Inc. All rights reserved.
This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 ACKNOWLEDGEMENTS

Thank you to the authors of my prequisite modules. With out your help this code
would be much more difficult to write!

 XML::Simple - Grant McLean
 XML::SAX    - Matt Sergeant
 DateTime    - Dave Rolsky

Also the authors of litmus, a very helpful tool indeed!

=head1 SEE ALSO

HTTP::DAV, HTTP::Webdav, http://www.webdav.org/, RFC 2518

=cut

1;


